
#include "version.h"

#define STR2(x)                 #x
#define STR(x)                  STR2(x)

#define APPNAME                 "TellTale"
#define APPEXTNAME              "TT"
#define APPANDVER               APPNAME " " STR(PVERSION)
#define APPANDVERANDDATE        APPANDVER " " __DATE__
#define APPFULLMONTY            APPANDVERANDDATE \
                                " - Tell-Tale Indicator by ChAoS@glod.net"

/*------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>

void    xprintf(        char            *fmt,
                        ... )
{
static  char    Message[128];

        va_list ap;

        va_start(ap,fmt);
        vsprintf(Message,fmt,ap);
        va_end(ap);

        OutputDebugString(Message);
}

void    xrect(          char            *name,
                        RECT            *R )
{
        xprintf("%s = (%ld,%ld)->(%ld,%ld)\n",
                name,R->left,R->top,R->right,R->bottom);
}

/*------------------------------------------------------------------------*/

#define CHAOS_UTILS_VERSION 1
#include "vping.cpp"

#define CANT_GET_IP_ADDRESS    ( -2 )

/*------------------------------------------------------------------------*/

#define DEFAULT_SIZE            ( 128 )
#define DEFAULT_FONT_SIZE       ( 10 )
#define DEFAULT_GAP             ( 0 )
#define DEFAULT_PERIOD          ( 500 )
#define DEFAULT_PING_TIMEOUT    ( 500 )
#define DEFAULT_WAIT            ( 10000 )

#define MAX_COLOURS             ( 20 )
#define MAX_POSITIONS           ( 50 )

/*------------------------------------------------------------------------*/

#define ZERO                    ( 0 )
#define ONE                     ( 1 )
#define TWO                     ( 2 )
#define THREE                   ( 3 )

#define NOT_SET                 ( -1 )

#define NILSTR                  ( "" )
#define SPACESTR                ( " " )
#define DOTSTR                  ( "." )
#define BACKSLASHSTR            ( "\\" )
#define DQUOTESTR               ( "\"" )

#define EOS                     ( '\0' )
#define TAB                     ( '\t' )
#define CARRIAGERETURN          ( '\r' )
#define LINEFEED                ( '\n' )
#define SPACE                   ( ' ' )
#define MINUS                   ( '-' )
#define PLUS                    ( '+' )
#define SLASH                   ( '/' )
#define SINGLEQUOTE             ( '\'' )
#define BACKSLASH               ( '\\' )
#define DOUBLEQUOTE             ( '"' )
#define PERCENT                 ( '%' )
#define COLON                   ( ':' )
#define SEMICOLON               ( ';' )
#define COMMA                   ( ',' )
#define EQUAL                   ( '=' )
#define EXCLAMATION             ( '!' )
#define ATSIGN                  ( '@' )
#define DOT                     ( '.' )

/*------------------------------------------------------------------------*/

#define RED                     ( 0x0000FF )
#define AMBER                   ( 0x2080FF )
#define GREEN                   ( 0x00FF00 )
#define BLACK                   ( 0x000000 )
#define WHITE                   ( 0xFFFFFF )
#define HIDDEN                  ( NOT_SET - ONE )

#define FLASH_TIMER             ( 1 )
#define PROG_TIMER              ( 2 )

#define MAX_CMDLINE_LEN         ( 8 * 1024 )

#define TOP_LEFT                ( 1 )
#define TOP_RIGHT               ( 2 )
#define BOTTOM_LEFT             ( 3 )
#define BOTTOM_RIGHT            ( 4 )

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

typedef enum    { OD_NONE, OD_UP, OD_DOWN, OD_LEFT, OD_RIGHT } OffDir;

typedef enum    { SCAN_ASSASSIN, SCAN_ARRANGE } ScanAction;

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

typedef struct  _ColourDef
{
        char    *Name;
        int     Value;
}       ColourDef;

static  ColourDef ColourNames[] =
{
        "AMBER",        AMBER,
        "BLACK",        BLACK,
        "BLUE",         0xFF0000,
        "CYAN",         0xFFFF00,
        "GRAY",         0x808080,
        "GREEN",        GREEN,
        "MAGENTA",      0xFF00FF,
        "RED",          RED,
        "WHITE",        WHITE,
        "YELLOW",       0x00FFFF,
        "HIDE",         HIDDEN
};

#define NOOFCOLOURDEFS  ( sizeof(ColourNames)/sizeof(ColourNames[ZERO]) )

/*------------------------------------------------------------------------*/

typedef struct  _ColourChange
{
        int     BoxColour;
        int     TextColour;
        int     Period;
}       ColourChange;

static  ColourChange    Colours[MAX_COLOURS];
static  int             NumColours = ZERO;

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

static  int     xPos                    = NOT_SET;
static  int     yPos                    = NOT_SET;

static  int     xSiz                    = DEFAULT_SIZE;
static  int     ySiz                    = DEFAULT_SIZE;

static  int     xGap                    = DEFAULT_GAP;
static  int     yGap                    = DEFAULT_GAP;

static  int     OffSetNum               = NOT_SET;
static  OffDir  OffSetDir               = OD_NONE;

static  int     Placement               = NOT_SET;

static  int     TimerPeriod             = DEFAULT_WAIT;

static  int     GroupNumber             = NOT_SET;

static  BOOL    TaskBar                 = FALSE;
static  BOOL    Maximised               = FALSE;
static  BOOL    Active                  = FALSE;
static  BOOL    KillAll                 = FALSE;

static  BOOL    IsPing                  = FALSE;
static  char    CmdLin[MAX_CMDLINE_LEN] = NILSTR;

static  char    BoxStr[MAX_CMDLINE_LEN] = NILSTR;

static  int     PingTimeOut             = NOT_SET;

static  int     FontSize                = NOT_SET;
static  char    FontName[MAX_CMDLINE_LEN] = NILSTR;

/*------------------------------------------------------------------------*/

static  int     ColourTimer             = ZERO;
static  int     ProgTimer               = ZERO;

static  int     LastErrLev              = ZERO;

static  char    OurClass[128]           = NILSTR;

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

static  int GLEnumber;

void    Box(            char            *message )
{
        GLEnumber = GetLastError();

        MessageBox(NULL,message,APPANDVERANDDATE,MB_OK);
}


void    BoxTxt(         char            *message,
                        char            *txt )
{
static  char Text[MAX_CMDLINE_LEN+ONE];

        GLEnumber = GetLastError();

        sprintf(Text,"%s: <%s>",message, txt==NULL ? "<null>" : txt);

        MessageBox(NULL,Text,APPANDVERANDDATE,MB_OK);
}


void    BoxInt(         char            *message,
                        int             value )
{
        char Text[128];

        GLEnumber = GetLastError();

        sprintf(Text,"%s: %d = 0x%X",message,value,value);
        MessageBox(NULL,Text,APPANDVERANDDATE,MB_OK);
}


void    BoxErrNo(       int             ErrNo )
{
        char Text[32];

        sprintf(Text,"Error number: %d = 0x%X",ErrNo,ErrNo);
        MessageBox(NULL,Text,APPANDVERANDDATE,MB_OK);
}

/*------------------------------------------------------------------------*/

/*      You must have previously called BOX(), BOXTXT() or BOXINT() so that
 *      GLENUMBER has already been set up with the value from GETLASTERROR().
 */

void    BoxErrGLE(      void )
{
        BoxInt("GetLastError",GLEnumber);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

/* Returns position of char C in string S.  Returns -1 if it can't find C. */

int     strpos(         char            *s,
                        char            c )
{
        int i;
        int len = strlen(s);

        for     ( i=ZERO ; i<len ; i++ )
                if ( s[i] == c ) return i;

        return -ONE;
}

/*------------------------------------------------------------------------*/

char    *strdel(        char            *s,
                        int             pos,
                        int             num )
{
        int     len = strlen(s);

        if      ( ( len > 0 ) && ( num > 0 ) && ( pos < len ) )
                if      ( num > ( len - pos + 1 ) )     /* Then deleting to */
                        s[pos] = EOS;                   /* the end of "s".  */
                else
                        strcpy(s+pos,s+pos+num);        /* Else just copy   */
                                                        /* the rest down.   */
        return  ( s );
}

/*------------------------------------------------------------------------*/

char    *strins(        char            *targ,
                        char            *src,
                        int             pos )
{
        char    *s;
        char    *t;
        int     lenT;
        int     lenS;
        int     i;

        lenT = strlen(targ), lenS = strlen(src);

        if ( ( pos < 0 ) || ( lenS <= 0 ) ) return targ;

        if      ( pos >= lenT )
        {
                strcat(targ,src);
                return targ;
        }

        s = targ + lenT;
        t = s + lenS;
        for( i=0; i<(lenT-pos+1); ++i ) *t-- = *s--;

        s = src;
        t = targ + pos;
        for ( i=0; i<lenS; ++i ) *t++ = *s++;

        return  ( targ );
}

/*------------------------------------------------------------------------*/

BOOL    IsPresent(      char            *s )
{
        return ( strlen(s) > ZERO );
}

/*------------------------------------------------------------------------*/

BOOL    IsSame(         char            *s,
                        char            *t )
{
        return  ( strcmp(s,t) == ZERO );
}

/*--------------------------------------------------------------------------*/

int     CharToHex( char c )
{
        if ( '0' <= c && c <= '9' ) return c - '0';
        if ( 'A' <= c && c <= 'F' ) return c - 'A'+ 10;
        if ( 'a' <= c && c <= 'f' ) return c - 'a'+ 10;
        return ZERO;
}

/*--------------------------------------------------------------------------*/

int     XlatBSlashes(  char    *s )
{
        char    *t = s;

        while   ( *t )
        {
                if      ( *t == BACKSLASH )
                {
                        strdel(t,ZERO,ONE);

                        switch  ( *t )
                        {
                        case '\0': *t = ' ';  break;
                        case 'a':  *t = 0x07; break;
                        case 'b':  *t = 0x08; break;
                        case 'e':  *t = 0x1B; break;
                        case 'f':  *t = 0x0C; break;
                        case 'n':  *t = 0x0A; break;
                        case 'r':  *t = 0x0D; break;
                        case 't':  *t = 0x09; break;
                        case 'v':  *t = 0x0B; break;

                        case 'x':
                                *t = ( CharToHex(t[1]) << 4 ) | CharToHex(t[2]);
                                strdel(t+ONE,ZERO,TWO);
                                break;
                        }
                }

                t++;
        }

        return  t - s;
}

/*------------------------------------------------------------------------*/

void    StripTrailChar( char            *s,
                        char            c )
{
        while   ( IsPresent(s) && ( s[strlen(s)-ONE] == c ) )
                strdel(s,strlen(s)-ONE,ONE);
}

/*------------------------------------------------------------------------*/

void    StripLeadChar(  char            *s,
                        char            c )
{
        while ( s[ZERO] == c ) strdel(s,ZERO,ONE);
}

/*------------------------------------------------------------------------*/

void    AllToUpper(     char            *s )
{
        while   ( *s )
        {
                *s = toupper(*s);
                s++;
        }
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/


/* Works out what the current value should be to pass to a SHOWWINDOW()
 * call.  It takes into account the command-line parameters and whether the
 * window should currently be hidden.
 */

int     WinStyle(       BOOL            Hidden )
{
        int     ShowStyle;

        if      ( Hidden )
                ShowStyle = SW_HIDE;
        else
        if      ( !Active )
                ShowStyle = SW_SHOWNA;
        else
                ShowStyle = SW_SHOW;

        return  ShowStyle;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

static  char Digits[] = "0123456789ABCDEF";

BOOL    atol2(          char            *s,
                        long            *value )
{
        long    Base  = 10;
        int     Pos;
        BOOL    Negative = FALSE;

        *value = ZERO;

        if      ( *s == MINUS )
        {
                Negative = TRUE;
                s++;
        }

        if      ( ( *s == '0' )
        &&        ( ( *(s+ONE) == 'X' ) || ( *(s+ONE) == 'x' ) ) )
                Base = 16, s += TWO;

        if      ( ( *(s+strlen(s)-ONE) == 'H' )
        ||        ( *(s+strlen(s)-ONE) == 'h' ) )
                Base = 16;

        while   ( *s )
        {
                Pos = strpos(Digits,(char)toupper(*s));

                if ( Pos < ZERO ) return FALSE;

                if ( ( Base == 10 ) && ( Pos > 9 ) ) return FALSE;

                *value = *value * Base + Pos;

                s++;
        }

        if ( Negative ) *value = -(*value);

        return  TRUE;
}

BOOL    atoi2(          char            *s,
                        int             *value )
{
        long    Lvalue;

        if ( !atol2(s,&Lvalue) ) return FALSE;

        *value = (int) Lvalue;

        return TRUE;
}

/*------------------------------------------------------------------------*/

BOOL    atoc(           char            *Colour,
                        int             *Value )
{
        int     i;

        AllToUpper(Colour);

        for     ( i=ZERO ; i<NOOFCOLOURDEFS ; i++ )
                if      ( IsSame(Colour,ColourNames[i].Name) )
                {
                        *Value = ColourNames[i].Value;
                        return TRUE;
                }

        if ( atoi2(Colour,Value) ) return TRUE;

        BoxTxt("Bad colour",Colour);
        return FALSE;
}

/*------------------------------------------------------------------------*/

BOOL    Get2Cols(       char            *s,
                        int             *x,
                        int             *y )
{
        char    *Xptr = s;
        char    *Yptr = NILSTR;
        BOOL    OK = TRUE;

        int     Pos = strpos(Xptr,COMMA);

        if ( Pos < ZERO ) Pos = strpos(Xptr,SEMICOLON);
        if ( Pos < ZERO ) Pos = strpos(Xptr,COLON);

        if ( Pos >= ZERO ) Xptr[Pos] = EOS, Yptr = Xptr + Pos + ONE;

        if ( IsPresent(Xptr) ) OK = OK && atoc(Xptr,x);
        if ( IsPresent(Yptr) ) OK = OK && atoc(Yptr,y);

        return OK;
}

/*------------------------------------------------------------------------*/

BOOL    Get2Nums(       char            *s,
                        int             *x,
                        int             *y )
{
        char    *Xptr = s;
        char    *Yptr = NILSTR;
        BOOL    OK = TRUE;

        int     Pos = strpos(Xptr,COMMA);

        if ( Pos < ZERO ) Pos = strpos(Xptr,SEMICOLON);
        if ( Pos < ZERO ) Pos = strpos(Xptr,COLON);

        if ( Pos >= ZERO ) Xptr[Pos] = EOS, Yptr = Xptr + Pos + ONE;

        if ( IsPresent(Xptr) ) OK = OK && atoi2(Xptr,x);
        if ( IsPresent(Yptr) ) OK = OK && atoi2(Yptr,y);

        return OK;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

static  int     ErrorLevel = NOT_SET;
static  int     CurrColour = ZERO;

/*------------------------------------------------------------------------*/

void    ZapColours(     void )
{
        int     i;

        for     ( i=ZERO ; i<MAX_COLOURS ; i++ )
                Colours[i].BoxColour  = NOT_SET,
                Colours[i].TextColour = NOT_SET,
                Colours[i].Period     = DEFAULT_PERIOD;
}

/*------------------------------------------------------------------------*/

void    SetErrorLevel(  int             CurrErrLev )
{
        ErrorLevel = CurrErrLev;
        CurrColour = ZERO;
}

/*------------------------------------------------------------------------*/

BOOL    SetPingLevel(   int             PingVal )
{
        int     UserColours = ( NumColours / TWO ) - ONE;
        int     n;

        if ( PingVal == CANT_GET_IP_ADDRESS ) return FALSE;

        if      ( PingVal < ZERO )
        {
                SetErrorLevel(UserColours);
                return TRUE;
        }

        n = ( UserColours * PingVal ) / PingTimeOut;
        if ( n >= UserColours ) n = UserColours - ONE;

        SetErrorLevel(n);
        return TRUE;
}

/*------------------------------------------------------------------------*/

void    NextColour(     int             *NextColour,
                        int             *NextText,
                        int             *NextPeriod )
{
        int     i = CurrColour;

        if      ( ErrorLevel != NOT_SET )
        {
                if      ( ( CurrColour & ONE ) == ZERO )
                {
                        i = TWO * ErrorLevel;
                        if ( i >= NumColours ) i = NumColours - TWO;
                        if ( i < ZERO ) i = ZERO;
                }
        }

        *NextColour = Colours[i].BoxColour;
        *NextText   = Colours[i].TextColour;
        *NextPeriod = Colours[i].Period;

        if ( ++i >= NumColours ) i = ZERO;

        CurrColour = i;
}

/*------------------------------------------------------------------------*/

BOOL    AddColour(      char            *Param )
{
        if ( NumColours >= MAX_COLOURS ) return FALSE;

        if      ( !Get2Cols(Param,&Colours[NumColours].BoxColour,
                                  &Colours[NumColours].TextColour) )
                return FALSE;

        ++NumColours;

        return  TRUE;
}

/*------------------------------------------------------------------------*/

BOOL    SetPeriod(      char            *Param )
{
        int     i,Period;

        if ( !atoi2(Param,&Period) ) return FALSE;

        for     ( i=NumColours ; i<MAX_COLOURS ; i++ )
                Colours[i].Period = Period;

        return  TRUE;
}

/*------------------------------------------------------------------------*/

BOOL    CheckColours(   void )
{
        if ( NumColours > ZERO ) return TRUE;

        if      ( IsPing )
                return  AddColour("GREEN,BLACK")
                &&      AddColour("GREEN,BLACK")
                &&      AddColour("AMBER,BLACK")
                &&      AddColour("AMBER,BLACK")
                &&      AddColour("RED,BLACK")
                &&      AddColour("RED,BLACK");

        return  AddColour("RED,BLACK")
        &&      AddColour("BLACK,BLACK");
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

BOOL    GetFont(        char            *Param )
{
        char    *Xptr = Param;
        char    *Yptr = NILSTR;
        BOOL    OK = TRUE;

        int     Pos = strpos(Xptr,COMMA);

        if ( Pos < ZERO ) Pos = strpos(Xptr,SEMICOLON);
        if ( Pos < ZERO ) Pos = strpos(Xptr,COLON);

        if ( Pos >= ZERO ) Xptr[Pos] = EOS, Yptr = Xptr + Pos + ONE;

        if ( IsPresent(Xptr) ) OK = OK && atoi2(Xptr,&FontSize);
        if ( IsPresent(Yptr) ) strcpy(FontName,Yptr);

        return OK;
}

/*------------------------------------------------------------------------*/

BOOL    GetPing(        char            *Param )
{
        char    *Xptr = Param;
        char    *Yptr = NILSTR;
        BOOL    OK = TRUE;

        int     Pos = strpos(Xptr,COMMA);

        if ( Pos < ZERO ) Pos = strpos(Xptr,SEMICOLON);
        if ( Pos < ZERO ) Pos = strpos(Xptr,COLON);

        if ( Pos >= ZERO ) Xptr[Pos] = EOS, Yptr = Xptr + Pos + ONE;

        if ( IsPresent(Xptr) ) strcpy(CmdLin,Xptr);
        if ( IsPresent(Yptr) ) OK = OK && atoi2(Yptr,&PingTimeOut);

        IsPing = TRUE;

        return OK;
}

/*------------------------------------------------------------------------*/

BOOL    GetOffSet(      char            *Param )
{
        switch  ( toupper(*Param++) )
        {
        case 'U': OffSetDir = OD_UP;    break;
        case 'D': OffSetDir = OD_DOWN;  break;
        case 'L': OffSetDir = OD_LEFT;  break;
        case 'R': OffSetDir = OD_RIGHT; break;
        default:  return FALSE;
        }

        if ( !IsPresent(Param) ) return TRUE;

        return  atoi2(Param,&OffSetNum);
}

/*------------------------------------------------------------------------*/

BOOLEAN DoParam(        char            *Parm )
{
        BOOL ArgOk = TRUE;

        if ( strlen(Parm) < TWO ) return FALSE;

        switch  ( toupper(Parm[ONE]) )
        {
        case 'A': Active = !Active;                                     break;
        case 'C': ArgOk = AddColour(Parm+TWO);                          break;
        case 'D': strcpy(BoxStr,Parm+TWO); XlatBSlashes(BoxStr);        break;
        case 'F': ArgOk = GetFont(Parm+TWO);                            break;
        case 'G': ArgOk = Get2Nums(Parm+TWO,&xGap,&yGap);               break;
        case 'I': ArgOk = SetPeriod(Parm+TWO);                          break;
        case 'K': KillAll = !KillAll;                                   break;
        case 'M': Maximised = !Maximised;                               break;
        case 'N': ArgOk = GetPing(Parm+TWO);                            break;
        case 'P': ArgOk = Get2Nums(Parm+TWO,&xPos,&yPos);               break;
        case 'O': ArgOk = GetOffSet(Parm+TWO);                          break;
        case 'S': ArgOk = Get2Nums(Parm+TWO,&xSiz,&ySiz);               break;
        case 'U': ArgOk = atoi2(Parm+TWO,&GroupNumber);                 break;
        case 'V': TaskBar = !TaskBar;                                   break;
        case 'W': ArgOk = atoi2(Parm+TWO,&TimerPeriod);                 break;
        case 'X': strcpy(CmdLin,Parm+TWO); IsPing = FALSE;              break;

        case 'B': if    ( toupper(Parm[TWO]) == 'L' )
                        Placement = BOTTOM_LEFT;
                  else
                  if    ( toupper(Parm[TWO]) == 'R' )
                        Placement = BOTTOM_RIGHT;
                  else
                        ArgOk = FALSE;
                  break;

        case 'T': if    ( toupper(Parm[TWO]) == 'L' )
                        Placement = TOP_LEFT;
                  else
                  if    ( toupper(Parm[TWO]) == 'R' )
                        Placement = TOP_RIGHT;
                  else
                        ArgOk = FALSE;
                  break;

        default:  ArgOk = FALSE;                                        break;
        }

        return ArgOk;
}

/*------------------------------------------------------------------------*/

BOOLEAN ParamStart(     char            *Cmd )
{
        return ( ( *Cmd == MINUS )
        ||       ( *Cmd == SLASH )
        ||       ( *Cmd == EQUAL )
        ||       ( *Cmd == PLUS ) );
}

BOOLEAN IsParam(        char            *Cmd )
{
        if ( ParamStart(Cmd) ) return TRUE;

        return ( ( *Cmd == DOUBLEQUOTE ) || ( *Cmd == SINGLEQUOTE ) )
        &&      ParamStart(Cmd+ONE);
}

/*------------------------------------------------------------------------*/

BOOLEAN FileExists(     char            *FileName )
{
        FILE *FD = fopen(FileName,"r");

        if ( FD == NULL ) return FALSE;

        fclose(FD);

        return TRUE;
}

/*------------------------------------------------------------------------*/

BOOLEAN SussFileName(   char            *GivenName,
                        char            *DefExtension )
{
static  char    Dir[MAX_PATH+ONE];
static  char    Name[MAX_PATH+ONE];
        char    *Slash;

        if ( FileExists(GivenName) ) return TRUE;

        strcpy(Name,GivenName);
        strcat(Name,DOTSTR);
        strcat(Name,DefExtension);

        if      ( FileExists(Name) )
        {
                strcpy(GivenName,Name);
                return TRUE;
        }

        if      ( ( GetModuleFileName(NULL,Dir,sizeof(Dir)-ONE) == ZERO )
        ||        ( (Slash=strrchr(Dir,BACKSLASH)) == NULL ) )
                return FALSE;

        Slash[ONE] = EOS;

        strcpy(Name,Dir);
        strcat(Name,GivenName);

        if      ( FileExists(Name) )
        {
                strcpy(GivenName,Name);
                return TRUE;
        }

        strcpy(Name,Dir);
        strcat(Name,GivenName);
        strcat(Name,DOTSTR);
        strcat(Name,DefExtension);

        if      ( FileExists(Name) )
        {
                strcpy(GivenName,Name);
                return TRUE;
        }

        return FALSE;
}

/*------------------------------------------------------------------------*/

BOOLEAN GetFileText(    char            *Name,
                        char            *Data )
{
        int     Next;
        FILE    *fd;

        if ( (fd=fopen(Name,"r")) == NULL ) return FALSE;

        while   ( (Next=fgetc(fd)) != EOF )
        {
                if      ( ( Next == TAB )
                ||        ( Next == CARRIAGERETURN )
                ||        ( Next == LINEFEED ) )
                        Next = SPACE;

                if ( isprint(Next) ) *Data++ = Next;
        }

        *Data = EOS;

        fclose(fd);

        return TRUE;
}

/*------------------------------------------------------------------------*/

BOOLEAN DoResponse(     char            *Cmd,
                        char            *OldCmd )
{
        int     Pos;
static  char    FileName[MAX_PATH+ONE];
static  char    FileText[MAX_CMDLINE_LEN+ONE];

        strcpy(Cmd,OldCmd);

        while   ( (Pos=strpos(Cmd,ATSIGN)) >= ZERO )
        {
                Cmd += Pos;

                if      ( Cmd[ONE] == ATSIGN )
                {
                        strdel(Cmd,ZERO,ONE);
                        Cmd++;
                        continue;
                }

                if ( strpos(Cmd+ONE,ATSIGN) < ZERO ) return TRUE;

                strdel(Cmd,ZERO,ONE);
                Pos = strpos(Cmd,ATSIGN);
                strdel(Cmd,Pos,ONE);

                strncpy(FileName,Cmd,Pos);
                FileName[Pos] = EOS;

                strdel(Cmd,ZERO,Pos);

                if      ( !SussFileName(FileName,APPEXTNAME) )
                {
                        BoxTxt("Couldn't find response file",FileName);
                        BoxErrGLE();
                        return FALSE;
                }

                if      ( !GetFileText(FileName,FileText) )
                {
                        BoxTxt("Couldn't read response file",FileName);
                        BoxErrGLE();
                        return FALSE;
                }

                strins(Cmd,FileText,ZERO);
        }

        return TRUE;
}

/*------------------------------------------------------------------------*/

BOOLEAN DoParams(       char            *Cmd,
                        char            *OldCmd )
{
        int     Pos;
        char    StrDelim;

        strcpy(Cmd,OldCmd);
        strcat(Cmd,SPACESTR);
        strcat(Cmd,SPACESTR);

        StripLeadChar(Cmd,SPACE);

        while   ( IsPresent(Cmd) )
        {
                StrDelim = Cmd[ZERO];

                if      ( ( StrDelim == DOUBLEQUOTE )
                ||        ( StrDelim == SINGLEQUOTE ) )
                {
                        strdel(Cmd,ZERO,ONE);
                        Pos = strpos(Cmd,StrDelim);
                        if      ( Pos < ZERO )
                                Pos = strlen(Cmd) - TWO;
                        else
                                strdel(Cmd,Pos,ONE);
                }
                else
                        Pos = strpos(Cmd,SPACE);

                Cmd[Pos] = EOS;

                if      ( IsSame(Cmd,"-") )
                {
                        if ( !AddColour("RED,BLACK") )   return FALSE;
                        if ( !AddColour("BLACK,BLACK") ) return FALSE;
                }
                else
                if      ( IsSame(Cmd,"=") )
                {
                        if ( !AddColour("AMBER,BLACK") ) return FALSE;
                }
                else
                if      ( IsSame(Cmd,"+") )
                {
                        if ( !AddColour("GREEN,BLACK") ) return FALSE;
                        if ( !AddColour("BLACK,BLACK") ) return FALSE;
                }
                else
                if      ( !DoParam(Cmd) )
                {
                        BoxTxt("Bad parameter",Cmd);
                        return FALSE;
                }

                strcpy(Cmd,Cmd+Pos+ONE);
                StripLeadChar(Cmd,SPACE);
        }

        StripTrailChar(Cmd,SPACE);
        return TRUE;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

BOOL    ProcFinished(   HANDLE          RunningProcess,
                        int             *ErrLev )
{
        DWORD ExitCode;


        if      ( !GetExitCodeProcess(RunningProcess,&ExitCode) )
        {
                Box("Failed to get exit code");
                return FALSE;
        }

        if      ( ExitCode == STILL_ACTIVE )
        {
                *ErrLev = STILL_ACTIVE;
                return TRUE;
        }

        fflush(NULL);

        *ErrLev = ExitCode;
        return TRUE;
}

/*------------------------------------------------------------------------*/

static  char    StartDir[MAX_PATH+ONE];

BOOL    DoCommands(     char            *Orders,
                        int             *ErrLev )
{
static  char                    ComSpec[MAX_PATH+ONE];
        BOOL                    OK;
        MSG                     Msg;
        STARTUPINFO             Start;
        PROCESS_INFORMATION     ProcInfo;
        SECURITY_ATTRIBUTES     Security;

        if      ( GetCurrentDirectory(sizeof(StartDir),StartDir) == ZERO )
        {
                Box("Can't get current (start) directory");
                return FALSE;
        }

        GetStartupInfo(&Start);

        Start.dwFlags     |= STARTF_USESHOWWINDOW;
        Start.wShowWindow  = SW_SHOWMINNOACTIVE;

        Security.nLength              = sizeof Security;
        Security.lpSecurityDescriptor = NULL;
        Security.bInheritHandle       = TRUE;

        fflush(NULL);

        if      ( !CreateProcess( NULL,
                                  Orders,
                                  &Security,            // Process security
                                  &Security,            // Thread security
                                  TRUE,                 // Inherit handles
                                  ZERO,                 // Creation flags
                                  NULL,                 // Environment
                                  StartDir,&Start,&ProcInfo ) )
        {
                BoxTxt("Process create failed",Orders);
                BoxErrGLE();
                return FALSE;
        }

        while   ( (OK=ProcFinished(ProcInfo.hProcess,ErrLev))
        &&        *ErrLev == STILL_ACTIVE )
        {
                Sleep(500);
                PeekMessage(&Msg,NULL,0,0,PM_NOREMOVE);
        }

        return OK;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

HFONT   MakeFont(       HDC             hdc )
{
        LOGFONT         lf;

        lf.lfHeight        = -MulDiv(FontSize,GetDeviceCaps(hdc,LOGPIXELSY),72);
        lf.lfWidth          = ZERO;
        lf.lfEscapement     = ZERO;
        lf.lfOrientation    = ZERO;
        lf.lfWeight         = FW_DONTCARE;
        lf.lfItalic         = FALSE;
        lf.lfUnderline      = FALSE;
        lf.lfStrikeOut      = FALSE;
        lf.lfCharSet        = DEFAULT_CHARSET;
        lf.lfOutPrecision   = OUT_CHARACTER_PRECIS;
        lf.lfClipPrecision  = CLIP_DEFAULT_PRECIS;
        lf.lfQuality        = DEFAULT_QUALITY;
        lf.lfPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;
        strcpy(lf.lfFaceName,FontName);

        return  CreateFontIndirect(&lf);
}

/*------------------------------------------------------------------------*/

BOOL    GetExecResult(  HWND            hwnd,
                        char            *Commands )
{
        BOOL    OK;

        if      ( !IsPing )
        {
                OK = DoCommands(Commands,&LastErrLev);
                if ( OK ) SetErrorLevel(LastErrLev);
        }
        else
        {
                LastErrLev = ping(Commands,PingTimeOut);
                OK = SetPingLevel(LastErrLev);
                if ( !OK ) BoxTxt("Cannot obtain host address",Commands);
        }

        if ( !OK ) SendMessage(hwnd,WM_CLOSE,ZERO,ZERO);

        return  OK;
}

/*------------------------------------------------------------------------*/

void    KillTimers(     void )
{
        if ( ColourTimer != ZERO ) KillTimer(NULL,ColourTimer);
        if ( ProgTimer   != ZERO ) KillTimer(NULL,ProgTimer);

        ColourTimer = ProgTimer = ZERO;
}

/*------------------------------------------------------------------------*/

LRESULT
CALLBACK WndProc(       HWND            hwnd,
                        UINT            iMsg,
                        WPARAM          wParam,
                        LPARAM          lParam )
{
static  int             NextC, NextT, NextP;
static  char            Formatted[MAX_CMDLINE_LEN];
static  BOOL            SeenDown = FALSE;

        RECT            rArea,rText;
        HDC             hdc;
        PAINTSTRUCT     ps;
        HBRUSH          hBrush;
        HFONT           OurFont;

        switch  ( iMsg )
        {
        case WM_CREATE:

                if      ( IsPresent(CmdLin) )
                {
                        if ( !GetExecResult(hwnd,CmdLin) ) return ZERO;

                        ProgTimer = SetTimer(hwnd,PROG_TIMER,TimerPeriod,NULL);
                }

                NextColour(&NextC,&NextT,&NextP);
                ColourTimer = SetTimer(hwnd,FLASH_TIMER,NextP,NULL);

                return ZERO;

        case WM_TIMER:

                switch  ( wParam )
                {
                case FLASH_TIMER:

                        NextColour(&NextC,&NextT,&NextP);

                        if ( ColourTimer != ZERO ) KillTimer(NULL,ColourTimer);
                        ColourTimer = SetTimer(hwnd,FLASH_TIMER,NextP,NULL);

                        ShowWindow(hwnd,WinStyle(NextC==HIDDEN));
                        InvalidateRect(hwnd,NULL,FALSE);

                        return ZERO;

                case PROG_TIMER:

                        if ( !GetExecResult(hwnd,CmdLin) ) KillTimers();
                        return ZERO;
                }

        case WM_PAINT:

                GetClientRect(hwnd,&rArea);

                hBrush = CreateSolidBrush(NextC);

                hdc = BeginPaint(hwnd,&ps);

                FillRect(hdc,&rArea,hBrush);

                if  ( IsPresent(BoxStr) )
                {
                     if         ( FontSize != NOT_SET )
                     {
                                OurFont = MakeFont(hdc);
                                SelectObject(hdc,OurFont);
                     }
                     SetTextColor(hdc,NextT);
                     SetBkMode(hdc,TRANSPARENT);
                     sprintf(Formatted,BoxStr,LastErrLev,LastErrLev,LastErrLev);
                     rText = rArea;
                     DrawText(hdc,Formatted,-1,&rText,DT_CENTER|DT_CALCRECT);

                     if       ( rText.bottom < rArea.bottom )
                              rText.top = ( rArea.bottom - rText.bottom ) / TWO,
                              rText.bottom += rText.top;

                     if       ( rText.right < rArea.right )
                              rText.left = ( rArea.right - rText.right ) / TWO,
                              rText.right += rText.left;

                     DrawText(hdc,Formatted,-1,&rText,DT_CENTER|DT_NOCLIP);
                }

                EndPaint(hwnd,&ps);

                if      ( IsPresent(BoxStr) && ( FontSize != NOT_SET ) )
                        DeleteObject(OurFont);

                DeleteObject(hBrush);

                return ZERO;

        case WM_LBUTTONDOWN:
        case WM_MBUTTONDOWN:
        case WM_RBUTTONDOWN:

                SeenDown = TRUE;
                return 0;

        case WM_LBUTTONUP:
        case WM_MBUTTONUP:
        case WM_RBUTTONUP:

                if ( !SeenDown ) return 0;      /* Ignore spurious release. */

                /* Fall through and commit suicide. */

        case WM_CHAR:
        case WM_DESTROY:

                KillTimers();
                PostQuitMessage(ZERO);
                return ZERO;
        }

        return DefWindowProc(hwnd,iMsg,wParam,lParam);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

static  ScannerOK = TRUE;

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

/* Kills off any sibling that is occupying the place that *we* want to be in.
 * On the other hand, if we are in "genocide" mode (that is, KILLALL is TRUE),
 * then *all* siblings (of the *same* group) will be killed.
 */

BOOL    Assassin(       HWND            hwnd )
{
        WINDOWPLACEMENT WP;

        if      ( !KillAll )
        {
                WP.length = sizeof(WP);

                if      ( !GetWindowPlacement(hwnd,&WP) )
                {
                        Box("Assassin can't get window placement");
                        ScannerOK = FALSE;
                        return FALSE;
                }

                /* If this sibling is not in *exactly* the same place as the
                 * new-born window, then we don't want to inhume this sibling.
                 */

                if      ( ( WP.rcNormalPosition.left   != xPos )
                ||        ( WP.rcNormalPosition.top    != yPos )
                ||        ( WP.rcNormalPosition.right  != ( xPos + xSiz ) )
                ||        ( WP.rcNormalPosition.bottom != ( yPos + ySiz ) ) )
                        return TRUE;

                /* The sibling is in the same *place* as the new-born window,
                 * but if it's not in the same "show state", we *don't* want to
                 * kill it.
                 */

                if      ( (  Maximised && ( WP.showCmd != SW_SHOWMAXIMIZED ) )
                ||        ( !Maximised && ( WP.showCmd == SW_SHOWMAXIMIZED ) ) )
                        return TRUE;
        }

        SendMessage(hwnd,WM_CLOSE,ZERO,ZERO);

        return TRUE;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

static  BOOL    InUse[MAX_POSITIONS];

/*------------------------------------------------------------------------*/

void    InitArranger( void )
{
        int     i;

        for     ( i=ZERO ; i<MAX_POSITIONS ; i++ )
                InUse[i] = FALSE;
}

/*------------------------------------------------------------------------*/

/* If this is called then
 *
 * (a) The user has specifed "arrange by offset" mode.
 * (b) They have *not* given us a *fixed* position to occupy.
 *
 * Therefore, we scan all windows and keep track of which windows are in the
 * specified arrangement pattern.  Tell-tales that we are interested in will
 * have:
 *
 * (a) The same size in bother the horizontal and vertical directions.
 *
 * (b) The same position in the "fixed" direction.  (For example, if the
 *     pattern is arranged vertically, then we will only look at tell-tales
 *     that match are required horizintal position.)
 *
 * Having scanned all the other tell-tales, we decided which is the first
 * non-used postion and alter OFFSETNUM to suite.  Later processing will take
 * care of actually setting the position globals.
 */

BOOL    Arranger(       HWND            hwnd )
{
        int     i;
        long    xThisSiz;       /* Size in X direction of sibling window.     */
        long    yThisSiz;       /* Size in Y direction of sibling window.     */
        long    ThisFixPos;     /* Pos of sibling in non-array direction.     */
        long    ThisVarPos;     /* Pos of sibling in array direction.         */
        long    NewArrGap;      /* New-born gap in array direction.           */
        long    NewFixPos;      /* New-born pos in non-array direction.       */
        long    NewArrSiz;      /* Size of new-born in the array direction.   */
        long    NewArrPos;      /* Base pos of slots in array direction.      */

        WINDOWPLACEMENT WP;

        WP.length = sizeof(WP);

        if      ( !GetWindowPlacement(hwnd,&WP) )
        {
                Box("Arranger can't get window placement");
                ScannerOK = FALSE;
                return FALSE;
        }

        /* If the sibling's not in the same "show state" as the new-born, we
         * aren't interested in it.
         */

        if      ( (  Maximised && ( WP.showCmd != SW_SHOWMAXIMIZED ) )
        ||        ( !Maximised && ( WP.showCmd == SW_SHOWMAXIMIZED ) ) )
                return TRUE;

        /* Work out the sizes of the sibling. */

        xThisSiz = WP.rcNormalPosition.right  - WP.rcNormalPosition.left;
        yThisSiz = WP.rcNormalPosition.bottom - WP.rcNormalPosition.top;

        /* If this sibling is not *exactly* the same size as the new-born
         * window, then we are not interested in it.
         */

        if ( ( xThisSiz != xSiz ) || ( yThisSiz != ySiz ) ) return TRUE;

        /* The sibing is the same *size* as the new-born, but is it in the
         * same *arrangement* as the new-born?
         *
         * First, we set up the fixed items of the new-born and sibling boxes.
         * Viz, the fixed position of the two boxes and the gap size of the
         * new-born.  We also select the size of the new-born in the direction
         * of which the array is aranged.
         */

        switch  ( OffSetDir )
        {
        case OD_UP:
        case OD_DOWN:

                ThisFixPos = WP.rcNormalPosition.left;
                ThisVarPos = WP.rcNormalPosition.top;
                NewFixPos  = xPos;
                NewArrGap  = yGap;
                NewArrSiz  = ySiz;
                NewArrPos  = yPos;

                break;

        case OD_LEFT:
        case OD_RIGHT:

                ThisFixPos = WP.rcNormalPosition.top;
                ThisVarPos = WP.rcNormalPosition.left;
                NewFixPos  = yPos;
                NewArrGap  = xGap;
                NewArrSiz  = xSiz;
                NewArrPos  = xPos;

                break;

        default:
                BoxInt("Arranger found invalid direction",(int)OffSetDir);
                ScannerOK = FALSE;
                return FALSE;
        }

        /* So, *are* they in the same arrangement? */

        if ( ThisFixPos != NewFixPos ) return TRUE;

        /* So, this sibling *is* in the same "arrangement" as the new-born.
         * We now "fiddle" the new-born's fixed size so that, in the comparison
         * below, we don't have to bother about which direction we're going in.
         */

        if      ( ( OffSetDir == OD_UP ) || ( OffSetDir == OD_LEFT ) )
                NewArrSiz = -NewArrSiz;

        /* We now have to see whether it in a position that would occupy one of
         * the tell-tale slots in the array.  Basically, we try all
         * combinations of OFFSETNUM until we either get to a number that's too
         * big or we find an exact match.
         */

        for     ( i=ZERO ; i<MAX_POSITIONS ; i++ )
        {
                /* If it *is* if the array, mark the slot as "in use" and
                 * return an "all OK" status.
                 */

                if      ( ThisVarPos == NewArrPos + ( i * NewArrSiz ) )
                {
                        InUse[i] = TRUE;
                        return TRUE;
                }
        }

        /* So, it's either not an *exact* match for one of the slots or it's
         * off of the end of the array.  Either way, we ignore it but return
         * an "all OK" status.
         */

        return TRUE;
}

/*------------------------------------------------------------------------*/

int     TermArranger( void )
{
        int     i;

        for     ( i=ZERO ; i<MAX_POSITIONS ; i++ )
                if ( !InUse[i] ) return i;

        return MAX_POSITIONS;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

BOOL CALLBACK Scanner(  HWND            hwnd,
                        LPARAM          ScannerAction )
{
static  char            Class[128+ONE];

        if      ( GetClassName(hwnd,Class,sizeof(Class)-ONE) == ZERO )
        {
                Box("Scanner can't get class info");
                ScannerOK = FALSE;
                return TRUE;
        }

        if ( !IsSame(Class,OurClass) ) return TRUE;    /* Not a sibling... */

        switch  ( ScannerAction )
        {
        case SCAN_ASSASSIN: return Assassin(hwnd);
        case SCAN_ARRANGE:  return Arranger(hwnd);
        }

        BoxInt("Invalid scanner action",(int)ScannerAction);
        ScannerOK = FALSE;
        return TRUE;
}

/*------------------------------------------------------------------------*/

BOOL    ScanClones(     ScanAction      Action )
{
        ScannerOK = TRUE;

        if ( Action == SCAN_ARRANGE ) InitArranger();

        if ( !EnumWindows(&Scanner,(LPARAM)Action) )
        {
                Box("Can't enumerate clones");
                return FALSE;
        }

        if      ( ScannerOK && ( Action == SCAN_ARRANGE ) )
                OffSetNum = TermArranger();

        return ScannerOK;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

/* Masage all parameters that haven't yet been set up. */

BOOL    InitGlobals(    void )
{
        int     xScreen = GetSystemMetrics(SM_CXSCREEN);
        int     yScreen = GetSystemMetrics(SM_CYSCREEN);

        if ( Placement == NOT_SET ) Placement = TOP_RIGHT;

        switch  ( Placement )
        {
        case TOP_LEFT:

                if ( xPos == NOT_SET ) xPos = xGap;
                if ( yPos == NOT_SET ) yPos = yGap;
                break;

        case TOP_RIGHT:

                if ( xPos == NOT_SET ) xPos = xScreen - xSiz - xGap;
                if ( yPos == NOT_SET ) yPos = yGap;
                break;

        case BOTTOM_LEFT:

                if ( xPos == NOT_SET ) xPos = xGap;
                if ( yPos == NOT_SET ) yPos = yScreen - ySiz - yGap;
                break;

        case BOTTOM_RIGHT:

                if ( xPos == NOT_SET ) xPos = xScreen - xSiz - xGap;
                if ( yPos == NOT_SET ) yPos = yScreen - ySiz - yGap;
                break;
        }

        if ( !CheckColours() ) return FALSE;

        if      ( ( OffSetDir != OD_NONE )
        &&        ( OffSetNum == NOT_SET )
        &&        !ScanClones(SCAN_ARRANGE) )
                return  FALSE;

        switch  ( OffSetDir )
        {
        case OD_UP:     yPos -= OffSetNum * ySiz; break;
        case OD_DOWN:   yPos += OffSetNum * ySiz; break;
        case OD_LEFT:   xPos -= OffSetNum * xSiz; break;
        case OD_RIGHT:  xPos += OffSetNum * xSiz; break;
        }

        if      ( IsPresent(FontName) && ( FontSize == NOT_SET ) )
                FontSize = DEFAULT_FONT_SIZE;

        if      ( PingTimeOut == NOT_SET )
                PingTimeOut = DEFAULT_PING_TIMEOUT;

        return TRUE;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

int WINAPI WinMain(     HINSTANCE       hInstance,
                        HINSTANCE       hPrevInstance,
                        PSTR            szCmdLine,
                        int             iCmdShow )
{
static  char            CmdLine1[MAX_CMDLINE_LEN+ONE];
static  char            CmdLine2[MAX_CMDLINE_LEN+ONE];
        HWND            hwnd;
        MSG             msg;
        WNDCLASSEX      wndclass;
        WSADATA         wsa;

        ZapColours();

        if ( !DoResponse(CmdLine1,szCmdLine) ) return EXIT_FAILURE;
        if ( !DoParams(CmdLine2,CmdLine1) )    return EXIT_FAILURE;

        if      ( GroupNumber == NOT_SET )
                strcpy(OurClass,APPANDVER);
        else
                sprintf(OurClass,"%s Group %d",APPNAME,GroupNumber);

        if  ( !InitGlobals() ) return EXIT_FAILURE;

        if ( !ScanClones(SCAN_ASSASSIN) ) return EXIT_FAILURE;

        if ( KillAll ) return ZERO;

        if ( IsPing ) WSAStartup(0x0101,&wsa);

        wndclass.cbSize        = sizeof (wndclass);
        wndclass.style         = CS_HREDRAW | CS_VREDRAW;
        wndclass.lpfnWndProc   = WndProc;
        wndclass.cbClsExtra    = ZERO;
        wndclass.cbWndExtra    = ZERO;
        wndclass.hInstance     = hInstance;
        wndclass.hIcon         = LoadIcon(hInstance,"TT_ICON");
        wndclass.hCursor       = LoadCursor(NULL,IDC_ARROW);
        wndclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
        wndclass.lpszMenuName  = NULL;
        wndclass.lpszClassName = OurClass;
        wndclass.hIconSm       = LoadIcon(hInstance,"TT_ICON");

        RegisterClassEx(&wndclass);

        hwnd = CreateWindowEx(
                     TaskBar ? WS_EX_TOPMOST : WS_EX_TOPMOST | WS_EX_TOOLWINDOW,
                     OurClass,
                     OurClass,
                     ( WS_POPUP & ~WS_DLGFRAME ) | WS_THICKFRAME,
                     xPos,yPos,
                     xSiz,ySiz,
                     NULL,NULL,
                     hInstance,NULL);

        if      ( Maximised )
                ShowWindow(hwnd,SW_SHOWMAXIMIZED);
        else
                ShowWindow(hwnd,WinStyle(Colours[ZERO].BoxColour==HIDDEN));

        UpdateWindow(hwnd);

        while   ( GetMessage(&msg,NULL,ZERO,ZERO) )
        {
                TranslateMessage(&msg);
                DispatchMessage(&msg);
        }

        return msg.wParam;
}

